#ifdef GL_ES
precision highp float;
#endif

#define NUM_OF_LAYERS 6.0
#define PI_OVER_SIX 0.523598

uniform vec2 u_resolution;
uniform float u_time;

mat2 Rot(float angle) {
    float s = sin(angle), c = cos(angle);
    return mat2(c, -s, s, c);
}
float Rand(vec2 seed) {
    seed = fract(seed * vec2(123.34, 456.21));
    seed += dot(seed, seed + 45.56);
    return fract(seed.x * seed.y);
}
float Rand(float seed) {
    seed = fract(seed * 12345.346);
    seed += dot(seed, seed + 45.5456);
    return fract(seed * seed);
}

float Star(vec2 uv, float flare) {
    float dist = length(uv);
    float radius = 0.025;
    
    float m = radius / dist;
    m += m * flare;
    
    // Adding cross rays
    float rays = max(0.0, 1.0 - abs(uv.x * uv.y * 1000.0));
    m += rays * flare;
    
    uv *= Rot(3.1415 / 4.0);
    
    // Adding slanted rays
    float slantedrays = max(0.0, 1.0 - abs(uv.x * uv.y * 2000.0));
    m += slantedrays * flare;
    
    // Adding glowing nebulae
    float nebula;
    nebula += Rand(fract(flare * 34.567));
    m += nebula * flare;
    
    m *= smoothstep(1.0, 0.2, dist);
    
    return m;
}
vec3 Starlayer(vec2 uv) {
    vec3 col = vec3(0.0);
    vec2 gv = fract(uv) - 0.5;
    vec2 id = floor(uv);
    
    for(int y = -1; y <= 1; y++) {
        for(int x = -1; x <= 1; x++) {
            vec2 offs = vec2(x, y);
            
            float n = Rand(id + offs);
            float size = fract(n * 345.32);
            float star = Star(gv - offs - (vec2(n, fract(n * 34.0)) - 0.5), smoothstep(0.9, 1.0, size) * 0.3);
            vec3 c = sin(vec3(0.3, 0.0, 1.0 + size) * fract(n * 230.54) * 10.2831) * 0.5 + 0.5;
            
            col += star * size * c;
        }
    }
    
    return col;
}
float Galaxy(vec2 uv, float flare) {
    float dist = length(uv);
    
    flare = smoothstep(0.01, 0.1, flare);
    vec2 uv1 = uv * Rot(3.1415 + Rand(6.283185) + Rand(flare));
    uv1 += Rand(flare * 3.1) + vec2(uv1.x + Rand(flare * 0.001), uv1.y - Rand(flare * 0.001));
    
    // Adding the galaxies
    float galaxy;
    float flareradius = 0.1 * flare;
    
    galaxy += smoothstep(flareradius, flareradius + 10.0, max(0.0, 1.0 - (10.0 * (uv1.x * uv1.x - 0.1) + 50.0 * (uv1.y * uv1.y))));
    galaxy += 3.5 * smoothstep(flareradius - 0.01, flareradius + 4.0, max(0.0, 1.0 - (10.0 * (uv1.x * uv1.x + 0.001) + 999.0 * (uv1.y * uv1.y))));
    galaxy += 1.1 * min(0.5, Rand(flare)) * smoothstep(0.001, flareradius + 2.0, max(0.0, 1.0 - (10.00 * (uv1.x * uv1.x + 0.01) + 100.0 * (uv1.y * uv1.y))));
    
    galaxy += galaxy * flare * 5.0;
    galaxy *= smoothstep(1.0, 0.2, dist);
    
    return galaxy;
}

vec3 Galaxylayer(vec2 uv) {
    vec3 col = vec3(0.0);
    vec2 gv = fract(uv) - 0.5;
    vec2 id = floor(uv);
    
    for(int y = -1; y <= 1; y++) {
        for(int x = -1; x <= 1; x++) {
            vec2 offs = vec2(x, y);
            
            float n = Rand(id + offs);
            float size = fract(n * 345.32);
            
            vec2 uv1 = gv - offs - (vec2(n, fract(n * 34.0)) - 0.5);
            uv1 = uv1 * Rot(3.1415 + Rand(6.283185 * n) + Rand(n));
            
            float galaxy = Galaxy(uv1, smoothstep(0.9, 1.0, size) * 0.3);
            vec3 c = sin(vec3(0.3, 0.0, 1.0 + size) * fract(n * 230.54) * 10.2831) * 0.5 + 0.5;
            
            col += galaxy * size * c;
        }
    }
    
    return col;
}

vec3 Quasar(vec2 uv, float intensity) {
    vec3 col;
    
    // Creating the outer debree field
    float d;
    float flareradius = 0.05;
    
    float dist = 1.0 / length(uv);
    
    d += sqrt(0.7 - abs(uv.x * uv.x) + abs(uv.y * uv.y)) + 0.1;
    d += smoothstep(0.001, 1.0, sqrt(0.7 - abs(uv.x * uv.x) + abs(uv.y * uv.y)) - 0.1);
    
    d *= d / smoothstep(flareradius, flareradius + 0.01, dist) * 0.28;
    d *= d / smoothstep(0.02, 0.02 + 0.01, dist);
    
    d *= 1.0 - smoothstep(1.0, 2.2, length(uv));
    
    col += max(0.001, d * 0.24) * vec3(0.99, 0.85, 0.5) * intensity;
    
    // Creating the disk
    float disk;
    disk += smoothstep(flareradius, flareradius + 2.0, max(0.0, 1.0 - (1.2 * (uv.x * uv.x + 0.1) + 10.0 * (uv.y * uv.y))));
    disk += smoothstep(flareradius, flareradius + 2.0, max(0.0, 1.0 - (0.25 * (uv.x * uv.x + 0.1) + 22.0 * (uv.y * uv.y))));
    vec3 diskcol = vec3(0.9, 0.42, 0.1) + disk;
    
    col += disk * diskcol;
    col += disk * diskcol * 1.5 * intensity;
    
    // Creating the star
    float m;
    float starradius = 0.045;
    float dist1;
    dist1 = length(uv);
    
    m += starradius / dist1 * 0.9;
    
    col += 0.001 - m * intensity;
    
    // Creating the gamma ray
    float p, q;
    p += max(0.0, smoothstep(0.001, 0.5, 0.1 - abs(uv.x * 0.5)));
    
    col += p * 8.0 * intensity;
    
    // Creating the gamma ray burst
    float s;
    float len = length(uv);
    len = sin(len * 5.0 - u_time * 20.0) / 15.0;
    
    float gammarayN, gammarayS;
    // North ray
    gammarayN = max(0.0, smoothstep(0.1, 3.0, 1.0 - ((uv.x * uv.x * 20.0) - (uv.y - len)) - 0.7));
    // South ray
    gammarayS = max(0.0, smoothstep(0.1, 3.0, 1.0 - ((uv.x * uv.x * 20.0) + (uv.y - len)) - 0.7));
    
    m += (gammarayN + gammarayS) * intensity;
    col += m * (gammarayN + gammarayS) * vec3(1.0, 1.0, 1.0) * ((sin(intensity * 5.0 + u_time) + sin(1234.0) + cos(1234.0) / 9999.0) + 0.0);
    vec3 c = (vec3(0.1, 0.01, 0.0 * intensity) * fract(230.54) * 10.0) * 0.5 + 0.5;
    col *= c;
    
    return col;
}

void main() {
    // Normalized pixel coordinates (from -1 to 1)
    vec2 uv = gl_FragCoord.xy / u_resolution.xy * 2.0 - 1.0;
    uv.x *= u_resolution.x / u_resolution.y;
    vec2 uv1 = uv * 1.5;
    
    vec3 color = vec3(0.0);
    float t = u_time * 0.01;
    uv1 *= Rot(PI_OVER_SIX);
    
    for(float i = 0.0; i < 1.0; i += 1.0 / NUM_OF_LAYERS) {
        float depth = fract(i + t);
        float scale = mix(20.0, 0.5, depth);
        float fade = depth * smoothstep(1.0, 0.9, depth);
        float layerfade = mix(0.001, 0.01, i);
        color += Starlayer(uv * scale + i * 453.2) * fade - layerfade;
    }
    color += Galaxylayer(uv * 4.0) / 5.0;
    color += Quasar(uv1, 1.0);
    
    gl_FragColor = vec4(color, 1.0);
}